import sys
import tornado.ioloop
import tornado.web
from tornado.options import define, options

from urls import urls
from settings import settings

define("port", default=9090, help="run on the given port", type=int)

print '\n\ninvoking server.py'

class Application(tornado.web.Application):
    print '\n\ninvoked Application()\n'
    def __init__(self):
        print '\ninside Application __init__()'
        tornado.web.Application.__init__(self, urls, **settings)
        print 'Aapplication __init__ caleed with web.Application()'
        print 'returning from Application __init__'


def main():
    tornado.options.parse_command_line()
    print '\ncreating app=Applicatoin()'
    app = Application()
    print '\napp= Aapplication() created -> app : ',app
    app.listen(options.port)
    print "\nStarting server on http://127.0.0.1:%s" % options.port

    try:
        tornado.ioloop.IOLoop.current().start()
    except KeyboardInterrupt:
        print "\nStopping server."






if __name__ == "__main__":
    main()


from tornado import web
from tornado.web import URLSpec as url
from sockjs.tornado import SockJSRouter

from settings import settings
from utils import include
from multiplex import MultiplexConnection
from apps.main.views import groupConnection, privateConnection
# Create multiplexer
router = MultiplexConnection.get(group=groupConnection, private=privateConnection)
print "router : ", router
# Register multiplexer
EchoRouter = SockJSRouter(router, '/chat')
# print "echorouter.ursl : ",EchoRouter.urls


urls = [
    url(r"/static/(.*)", web.StaticFileHandler, {"path": settings.get('static_path')}),
]
urls += include(r"/", "apps.main.urls")

urls = urls + EchoRouter.urls






import tornado.web
import tornado.escape

from sockjs.tornado import SockJSConnection

print '\n\ninvoked apps.main.views.py\n'

class IndexHandler(tornado.web.RequestHandler):
    """Regular HTTP handler to serve the chatroom page"""
    print "\n\ninvoked IndexHandler\n"
    def get(self):
    	print "\ninside IndexHandler get()"
        self.render('index.html')
        



# Connections
print "current Participant :"
participants=set()
print participants
print

class groupConnection(SockJSConnection):
	print '\n\nInvoked groupConnection\n'
	def on_open(self, info):
		print '\ninside groupConnection on_open()'
		print  "current participants : ",participants
		participants.add(self)
		print "updated participants : ",participants
		print 'returning from groupConnection on_open'

	def on_message(self, message):
		print '\ninside groupConnection on_message()'
		print 'self participant : ',self
		print  "current participants : ",participants
		print "message sent : ",message
		res = tornado.escape.json_decode(message)
		print "json_decode message : ",res
		if res['stage']=='start':
			print 'inside res[stage]==start'
			print "broadcasting... ***** name joined ****"
			self.broadcast(participants, '***** '+res['name']+' joined *****')
		elif res['stage']=='end':
			# print  "current participants : ",participants
			print 'inside res[stage]==end'
			print "broadcasting... ***** name joined ****"
			self.broadcast(participants, '***** '+res['name']+' left *****')
			print 'invoking participant.revove(self)'
			participants.remove(self)
			print 'updated participants : ',participants

		else:
			print 'inside res[stage]==process'
			print 'broadcating... [name] message'
			self.broadcast(participants, '['+res['name']+']  ' + res['message'])
		print 'returning from groupConnection on_message'

	def on_close(self,name):
		print '\ninside groupConnection on_message()'
		print  "current participants : ",participants
		participants.remove(self)
		print "message sent : ",message
		print 'broadcastting.... ***** name left ****'
		self.broadcast(participants,'*** '+name+' left ***')
		print 'returning from groupConnection on_close'


	    
	    

	

class privateConnection(SockJSConnection):
    def on_open(self, info):
        self.send('Carl says goodbye!')
        participants.add(self)

    def on_message(self,message):
	    pass

	
    def on_close(self):
	    participants.remove(self)
	    self.broadcast(participants,'Carl Left')





# !/usr/bin/env python
#
# setup.py script
#
# copyright 2016 anirban roy das <anirban.nick@gmail.com>
#
#

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
import  codecs
import os


############### general config ##############


NAME = "sockjsChat"

VERSION = '1.1.0'

# PACKAGES = find_packages(where="sockjsChat")
PACKAGES = ["sockjsChat", "sockjsChat.apps", "sockjsChat.apps.main"]

PROJECT_URL = 'https://github.com/anirbanroydas/sockjsChat'

AUTHOR = 'Anirban Roy Das'

EMAIL = 'anirban.nick@gmail.com'

KEYWORDS = ["chat server sockjs websocket tornado"]

CLASSIFIERS = [

    # How mature is this project? Common values are
	#   3 - Alpha
	#   4 - Beta
    #   5 - Production/Stable
	'Development Status :: 4 - Beta',

	# Indicate who your project is intended for
	'Intended Audience :: Developers',
	
    # Pick your license as you wish (should match "license" above)
	'License :: OSI Approved :: MIT License',

	# Specify the Natural Language 
	'Natural Language :: English',

	# Specify the operating systems it can work on 
	'Operating System :: OS Independent',
	# Specify the Python versions you support here. In particular, ensure that you indicate whether you support Python 2, Python 3 or both.
	'Programming Language :: Python',
	'Programming Language :: Python :: 2',
	'Programming Language :: Python :: 2.7',
	'Programming Language :: Python :: Implementation :: CPython',

	]

INSTALL_REQUIRES = ["tornado >= 2.2.1",
                    "sockjs-tornado",
                    "setuptools >= 0.7.0",]

EXTRAS_REQUIRE = {}

PACKAGE_DATA = {
		# data files need to be listed both here (which determines what gets
        # installed) and in MANIFEST.in (which determines what gets included
        # in the sdist tarball)
        "sockjsChat":["static/css/*.css",
        			  "static/js/*.js",
        			  "templates/*.html",
        			  ],

        }

# DATA_FILES =[]

HERE = os.path.abspath(os.path.dirname(__file__))

#############  End of basic config ###########



####  Get the long description from the README file
with codecs.open(os.path.join(HERE, 'README.rst'), 'rb' , 'utf-8') as f:
	LONG_DESCRIPTION = f.read()




#### The main setup function ######
setup(
    name=NAME,

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=VERSION,

    description='A Chat Server based on sockjs, websocket and tornado',
    long_description=LONG_DESCRIPTION,

    # The project's main homepage.
    url=PROJECT_URL,

    # Author details
    author=AUTHOR,
    author_email=EMAIL,

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=CLASSIFIERS,

    # What does your project relate to?
    keywords=KEYWORDS,

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=PACKAGES,

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=INSTALL_REQUIRES,

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require=EXTRAS_REQUIRE,

    # If set to True, this tells setuptools to automatically include 
    # any data files it finds inside your package directories that 
    # are specified by your MANIFEST.in file. 
    # include_package_data = True


    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data=PACKAGE_DATA,

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=DATA_FILES,

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'sockjsChat=sockjsChat.server:main',
        ],
    },
)


from sockjs.tornado import conn, session
from sockjs.tornado.transports import base

print '\n\ninvoked multiplex.py\n'



class ChannelSession(session.BaseSession):
    print '\n\nInvoked ChannelSession\n'
    def __init__(self, conn, server, base, name):
        print '\ninside ChannelSession __init__()'
        print 'conn : ',conn
        print 'server : ',server
        print 'base : ',base
        print 'name : ',name
        print "invoking super(ChannelSession,self).__init__(conn,server)"
        super(ChannelSession, self).__init__(conn, server)
        print 'super(ChannelSession,self).__init__(conn,server) invoked successfully'

        self.base = base
        self.name = name
        print 'self.base : ',self.base
        print 'self.name : ',self.name
        print 'returning from ChannelSession __init__()'

    def send_message(self, msg, stats=True, binary=False):
        print '\ninside ChannelSession send_message()'
        # TODO: Handle stats
        print 'self.base : ',self.base
        print 'self.name : ',self.name
        print 'msg : ',msg
        print "invoking self.base.send(msg+self.name,msg)"
        self.base.send('msg,' + self.name + ',' + msg)
        print "self.base.send(msg+self.name,msg) invoked successfully"
        print 'returning form ChannelSession send_message()'

    def on_message(self, msg):
        print '\ninside ChannelSession on_message()'
        print 'self.conn : ',self.conn
        print 'msg : ',msg
        print 'invoking self.conn.on_message(msg)'
        self.conn.on_message(msg)
        # self.base.send_message
        print 'self.base.on_message(msg) invoked successfully'
        print 'returning from ChannelSession on_message()'

    def on_close(self, code=3000, message='Go away!'):
        print '\ninside ChannelSession on_close()'
        print 'self.base : ',self.base
        print 'self.name : ',self.name
        print 'self.conn : ',self.conn
        print 'invoking self.conn.on_close(msg)'
        self.conn.on_close(msg)
        print ''
        print 'invoking self.base.send(uns + self.name)'
        self.base.send('uns,' + self.name)
        print 'self.base.send(uns + self.name) invoked successfully'
        print 'code : ',code
        print 'message : ',message
        print 'invoking self._close(code,message)'
        self._close(code, message)
        print 'self._close(code,message) invoked successfully'
        print 'returning from ChannelSession on_close'

    # Non-API version of the close, without sending the close message
    def _close(self, code=3000, message='Go away!'):
        print '\ninside ChannelSession _close()'
        print 'code : ',code
        print 'message : ',message
        print 'invoking super(ChannelSession,self).close(code,message)'
        super(ChannelSession, self).close(code, message)
        print 'super(ChannelSession,self).close(code,message) invoked successfully'
        print 'returning from ChannelSession _close()'


class DummyHandler(base.BaseTransportMixin):

    print '\n\n invoked DummyHandler\n'
    name = 'multiplex'


    def __init__(self, conn_info):
        print '\ninside DummyHandler __init__()'
        self.conn_info = conn_info
        print 'self.conn_info : ',self.conn_info
        print 'returning from DummyHandler __init__'

    def get_conn_info(self):
        print '\ninside DummyHandler get_conn_info()'
        print "returning self.conn_info : ",self.conn_info
        return self.conn_info


class MultiplexConnection(conn.SockJSConnection):
    print '\n\nInvoked multiplex.MultiplexConnection\n'
    print "current channels : "
    channels = dict()
    print channels

    def on_open(self, info):
        print '\ninside MultiplexConnection on_open'
        self.endpoints = dict()
        print "endpoints : ",self.endpoints
        self.handler = DummyHandler(self.session.conn_info)
        print " self.handler : ",self.handler
        print 'returning from MultiplexConnection on_open'

    def on_message(self, msg):
        
        print "\ninside MultiplexConnection on_message function"
	
    	print " pre parts msg : ",msg
    	parts = msg.split(',', 2)
    	print "post Parts  msg : ",parts
        op, chan = parts[0], parts[1]

        print 'op : ',op
        print 'chan : ',chan

        print "current channels : ",self.channels
        print 'current endpoints : ',self.endpoints

        if chan not in self.channels:
            print ' chan not in self.channels'
            return

        if chan in self.endpoints:
            print 'chan in self.endpoints'
            session = self.endpoints[chan]
            print 'session : ',session

            if op == 'uns':
                print 'op==uns'
                print 'current self.endpoints : ',self.endpoints
                del self.endpoints[chan]
                print 'updates self.endpoints : ',self.endpoints
                print 'invoking session._close()'
                session._close()
                print 'session._close invokes sucsessfully'
            elif op == 'msg':
                print 'op==msg'
                print 'current self.endpoints : ',self.endpoints
                print "current parts : ",parts
                print 'invoking session.on_message(parts[2])'
                session.on_message(parts[2])
                print 'session.on_message(parts[2]) invokes sucsessfully'
        else:
            print 'chan not in self.endpoints'
            if op == 'sub':
                print 'op==sub'
                print 'self.channels[chan] : ',self.channels[chan]
                print 'self.session.server : ',self.session.server
                print 'chan : ',chan
                print 'invokeing session=ChannelSession'
                session = ChannelSession(self.channels[chan],
                                         self.session.server,
                                         self,
                                         chan)
                print 'invoked session=channelSession successfully'
                print 'session : ',session
                print 'setting session.set_handler(self.handler)'
                session.set_handler(self.handler)
                print 'session.set_handler(self.handler) invoked successfully'
                print 'invoking session.verify_state()'
                session.verify_state()
                print 'session.verify_state invoked successfully'

                self.endpoints[chan] = session
        print 'returning from MultiplexConnection on_message'

    def on_close(self):
        print "\ninside MultiplexConnection on_close function"
        print 'current self.endpoints : ',self.endpoints
        print 'starting for loop : for chan in self.endpoints'
        for chan in self.endpoints:
            print 'chan : ',chan
            print 'self.endpoints[chan] : ',self.endpoints[chan]
            print 'invoking self.endpoints[chan]._close()'
            self.endpoints[chan].on_close()
        print 'returning from MultiplexConnection on_close'

    @classmethod
    def get(cls, **kwargs):
        print '\n inside MultiplexConnection @classmethod get()'
        res= type('MultiplexRouter', (MultiplexConnection,), dict(channels=kwargs))
        print 'returning value is : ',res
        return res
